package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.common.collect.Lists;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.BaseCustomerIndustry;
import org.jeecg.modules.basedata.model.BaseCustomerIndustryTree;
import org.jeecg.modules.basedata.service.IBaseCustomerIndustryService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 客商行业
 * @author： jeecg-boot
 * @date：   2021-11-11
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/baseCustomerIndustry")
@Slf4j
public class BaseCustomerIndustryController {
	@Autowired
	private IBaseCustomerIndustryService baseCustomerIndustryService;
	 @Autowired
	 private ISysSerialPatternService sysSerialPatternService;
	
	/**
	  * 分页列表查询
	 * @param baseCustomerIndustry
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<BaseCustomerIndustry>> queryPageList(BaseCustomerIndustry baseCustomerIndustry,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<BaseCustomerIndustry>> result = new Result<IPage<BaseCustomerIndustry>>();
		QueryWrapper<BaseCustomerIndustry> queryWrapper = QueryGenerator.initQueryWrapper(baseCustomerIndustry, req.getParameterMap());
		Page<BaseCustomerIndustry> page = new Page<BaseCustomerIndustry>(pageNo, pageSize);
		IPage<BaseCustomerIndustry> pageList = baseCustomerIndustryService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 查询客商行业树形数据
	* @Date 2021/11/11 19:29
	* @Param [pkOrg]
	* @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.basedata.model.BaseCustomerIndustryTree>>
	*/
	 @GetMapping(value = "/getTreeList")
	 public Result<List<BaseCustomerIndustryTree>> getTreeList(String pkOrg) {
		 Result<List<BaseCustomerIndustryTree>> result = new Result<>();
		 QueryWrapper<BaseCustomerIndustry> queryWrapper = new QueryWrapper<>();
		 queryWrapper.lambda().eq(BaseCustomerIndustry::getDelFlag,'0');
		 queryWrapper.lambda().eq(BaseCustomerIndustry::getPkOrg,pkOrg);
		 queryWrapper.lambda().orderByAsc(BaseCustomerIndustry::getSort);
		 List<BaseCustomerIndustry> list = baseCustomerIndustryService.list(queryWrapper);
		 List<BaseCustomerIndustryTree> listTree=getTree(list);
		 result.setSuccess(true);
		 result.setResult(listTree);
		 return result;
	 }

	 //获取客商行业树形数据
	 public List<BaseCustomerIndustryTree> getTree(List<BaseCustomerIndustry> list){
		 List<BaseCustomerIndustryTree> resp = Lists.newArrayList();
		 //根据parentId分组
		 Map<String, List<BaseCustomerIndustry>> postMap = list.stream().collect(Collectors.groupingBy(BaseCustomerIndustry :: getParentId));
		 List<BaseCustomerIndustry>  baseArchivesProjectApprovals = postMap.get("0");
		 if(CollectionUtils.isEmpty(baseArchivesProjectApprovals)){
			 return null;
		 }
		 for(BaseCustomerIndustry baseCustomerIndustry : baseArchivesProjectApprovals){
			 BaseCustomerIndustryTree baseCustomerIndustryTree = new BaseCustomerIndustryTree(baseCustomerIndustry);
			 baseCustomerIndustryTree.setChildren(getChildes(list, baseCustomerIndustryTree.getKey(), postMap));
			 //判断是否为末节点
			 List<BaseCustomerIndustry> posts = postMap.get(baseCustomerIndustryTree.getKey());
			 if(CollectionUtils.isEmpty(posts)){
				 baseCustomerIndustryTree.setIsLeaf(true);
				 baseCustomerIndustryTree.setChildren(null);
			 }
			 resp.add(baseCustomerIndustryTree);
		 }
		 return resp;
	 }

	 /**
	  * @desc 获取子节点数据
	  */
	 private List<BaseCustomerIndustryTree> getChildes(List<BaseCustomerIndustry> list, String key, Map<String, List<BaseCustomerIndustry>> postMap) {
		 List<BaseCustomerIndustryTree> resp = Lists.newArrayList();
		 for(BaseCustomerIndustry baseCustomerIndustry : list){
			 if(baseCustomerIndustry.getParentId().equals(key)){
				 BaseCustomerIndustryTree baseArchivesProjectApprovalRespDTO = new BaseCustomerIndustryTree(baseCustomerIndustry);
				 baseArchivesProjectApprovalRespDTO.setChildren(getChildes(list, baseArchivesProjectApprovalRespDTO.getKey(), postMap));
				 //判断是否为末节点
				 List<BaseCustomerIndustry> posts = postMap.get(baseArchivesProjectApprovalRespDTO.getKey());
				 if(CollectionUtils.isEmpty(posts)){
					 baseArchivesProjectApprovalRespDTO.setIsLeaf(true);
					 baseArchivesProjectApprovalRespDTO.setChildren(null);
				 }
				 resp.add(baseArchivesProjectApprovalRespDTO);
			 }
		 }
		 return resp;
	 }
	
	/**
	  *   添加
	 * @param baseCustomerIndustry
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<BaseCustomerIndustry> add(@RequestBody BaseCustomerIndustry baseCustomerIndustry) {
		Result<BaseCustomerIndustry> result = new Result<BaseCustomerIndustry>();
		try {
			// 设置编码
			CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(baseCustomerIndustry, "base_customer_industry", "code", true);
			if(!nextSerial.isSucceed()){
				throw new RuntimeException("获取编号失败");
			}
			baseCustomerIndustry.setCode(nextSerial.getContent());
			baseCustomerIndustryService.save(baseCustomerIndustry);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param baseCustomerIndustry
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<BaseCustomerIndustry> edit(@RequestBody BaseCustomerIndustry baseCustomerIndustry) {
		Result<BaseCustomerIndustry> result = new Result<BaseCustomerIndustry>();
		BaseCustomerIndustry baseCustomerIndustryEntity = baseCustomerIndustryService.getById(baseCustomerIndustry.getId());
		if(baseCustomerIndustryEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = baseCustomerIndustryService.updateById(baseCustomerIndustry);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<BaseCustomerIndustry> delete(@RequestParam(name="id",required=true) String id) {
		Result<BaseCustomerIndustry> result = new Result<BaseCustomerIndustry>();
		BaseCustomerIndustry baseCustomerIndustry = baseCustomerIndustryService.getById(id);
		if(baseCustomerIndustry==null) {
			result.error500("未找到对应实体");
		}else {
			baseCustomerIndustry.setDelFlag("1");
			boolean ok = baseCustomerIndustryService.updateById(baseCustomerIndustry);
			if(ok) {
				result.success("删除成功!");
			}
		}
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<BaseCustomerIndustry> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<BaseCustomerIndustry> result = new Result<BaseCustomerIndustry>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.baseCustomerIndustryService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<BaseCustomerIndustry> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BaseCustomerIndustry> result = new Result<BaseCustomerIndustry>();
		BaseCustomerIndustry baseCustomerIndustry = baseCustomerIndustryService.getById(id);
		if(baseCustomerIndustry==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(baseCustomerIndustry);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<BaseCustomerIndustry> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              BaseCustomerIndustry baseCustomerIndustry = JSON.parseObject(deString, BaseCustomerIndustry.class);
              queryWrapper = QueryGenerator.initQueryWrapper(baseCustomerIndustry, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<BaseCustomerIndustry> pageList = baseCustomerIndustryService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "客商行业列表");
      mv.addObject(NormalExcelConstants.CLASS, BaseCustomerIndustry.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("客商行业列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<BaseCustomerIndustry> listBaseCustomerIndustrys = ExcelImportUtil.importExcel(file.getInputStream(), BaseCustomerIndustry.class, params);
              for (BaseCustomerIndustry baseCustomerIndustryExcel : listBaseCustomerIndustrys) {
                  baseCustomerIndustryService.save(baseCustomerIndustryExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listBaseCustomerIndustrys.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
