package org.jeecg.modules.basedata.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.BaseMaterialFileClassification;
import org.jeecg.modules.basedata.model.BaseMaterialFileClassificationTree;
import org.jeecg.modules.basedata.service.IBaseMaterialFileClassificationService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 物料档案分类
 * @author： jeecg-boot
 * @date：   2021-11-16
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/baseMaterialFileClassification")
@Slf4j
public class BaseMaterialFileClassificationController {
	@Autowired
	private IBaseMaterialFileClassificationService baseMaterialFileClassificationService;
	
	/**
	  * 分页列表查询
	 * @param baseMaterialFileClassification
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<BaseMaterialFileClassification>> queryPageList(BaseMaterialFileClassification baseMaterialFileClassification,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<BaseMaterialFileClassification>> result = new Result<IPage<BaseMaterialFileClassification>>();
		QueryWrapper<BaseMaterialFileClassification> queryWrapper = QueryGenerator.initQueryWrapper(baseMaterialFileClassification, req.getParameterMap());
		Page<BaseMaterialFileClassification> page = new Page<BaseMaterialFileClassification>(pageNo, pageSize);
		IPage<BaseMaterialFileClassification> pageList = baseMaterialFileClassificationService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 获取物料分类树形
	* @Date 2021/11/16 14:24
	* @Param [pkOrg]
	* @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.basedata.model.BaseMaterialFileClassificationTree>>
	*/
	 @GetMapping(value = "/getTreeList")
	 public Result<List<BaseMaterialFileClassificationTree>> getTreeList(String pkOrg) {
		 Result<List<BaseMaterialFileClassificationTree>> result = new Result<>();
		 try {
			 QueryWrapper<BaseMaterialFileClassification> queryWrapper = new QueryWrapper<>();
			 queryWrapper.lambda().eq(BaseMaterialFileClassification::getPkOrg,pkOrg);
			 queryWrapper.lambda().eq(BaseMaterialFileClassification::getDelFlag,"0");
			 queryWrapper.lambda().orderByAsc(BaseMaterialFileClassification::getSort);
			 List<BaseMaterialFileClassification> list = baseMaterialFileClassificationService.list(queryWrapper);
			 List<BaseMaterialFileClassificationTree> treeList=new ArrayList<>();
			 getTree(treeList,list,null);
			 findLeaf(treeList);
			 result.setResult(treeList);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

	 //处理树形格式
	 private void getTree(List<BaseMaterialFileClassificationTree> treeList, List<BaseMaterialFileClassification> fdlist, BaseMaterialFileClassificationTree temp) {
		 int i = 0;
		 for (BaseMaterialFileClassification baseMaterialFileClassification : fdlist) {
			 String tempPid = baseMaterialFileClassification.getParentId();
			 BaseMaterialFileClassificationTree tree = new BaseMaterialFileClassificationTree(baseMaterialFileClassification);
			 if(temp == null) {
				 if(oConvertUtils.isEmpty(tempPid)){
					 treeList.add(tree);
				 }
				 if(!tree.getIsLeaf()) {
					 getTree(treeList, fdlist, tree);
				 }
			 }else if(tempPid != null && tempPid.equals(temp.getKey())){
				 temp.getChildren().add(tree);
				 if(!tree.getIsLeaf()) {
					 getTree(treeList, fdlist, tree);
				 }
			 }

		 }
	 }

	 //处理尾端
	 private void findLeaf(List<BaseMaterialFileClassificationTree> treeList){
		 for(BaseMaterialFileClassificationTree tree : treeList){
			 if(tree.getChildren().isEmpty()){
				 tree.setIsLeaf(true);
			 }else{
				 findLeaf(tree.getChildren());
			 }
		 }

	 }
	
	/**
	  *   添加
	 * @param baseMaterialFileClassification
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<BaseMaterialFileClassification> add(@RequestBody BaseMaterialFileClassification baseMaterialFileClassification) {
		Result<BaseMaterialFileClassification> result = new Result<BaseMaterialFileClassification>();
		try {
			baseMaterialFileClassificationService.save(baseMaterialFileClassification);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param baseMaterialFileClassification
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<BaseMaterialFileClassification> edit(@RequestBody BaseMaterialFileClassification baseMaterialFileClassification) {
		Result<BaseMaterialFileClassification> result = new Result<BaseMaterialFileClassification>();
		BaseMaterialFileClassification baseMaterialFileClassificationEntity = baseMaterialFileClassificationService.getById(baseMaterialFileClassification.getId());
		if(baseMaterialFileClassificationEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = baseMaterialFileClassificationService.updateById(baseMaterialFileClassification);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
//	/**
//	  *   通过id删除
//	 * @param id
//	 * @return
//	 */
//	@DeleteMapping(value = "/delete")
//	public Result<BaseMaterialFileClassification> delete(@RequestParam(name="id",required=true) String id) {
//		Result<BaseMaterialFileClassification> result = new Result<BaseMaterialFileClassification>();
//		BaseMaterialFileClassification baseMaterialFileClassification = baseMaterialFileClassificationService.getById(id);
//		if(baseMaterialFileClassification==null) {
//			result.error500("未找到对应实体");
//		}else {
//			boolean ok = baseMaterialFileClassificationService.removeById(id);
//			if(ok) {
//				result.success("删除成功!");
//			}
//		}
//
//		return result;
//	}


	 /**
	  *   通过id删除
	  * @param id
	  * @return
	  */
	 @DeleteMapping(value = "/delete")
	 public Result<BaseMaterialFileClassification> delete(@RequestParam(name="id",required=true) String id) {
		 Result<BaseMaterialFileClassification> result = new Result<BaseMaterialFileClassification>();
		 BaseMaterialFileClassification baseMaterialFileClassification = baseMaterialFileClassificationService.getById(id);
		 if(baseMaterialFileClassification==null) {
			 result.error500("未找到对应实体");
		 }else {
			 baseMaterialFileClassification.setDelFlag("1");
			 boolean ok = baseMaterialFileClassificationService.updateById(baseMaterialFileClassification);
			 if(ok) {
				 result.success("删除成功!");
			 }
		 }

		 return result;
	 }

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<BaseMaterialFileClassification> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<BaseMaterialFileClassification> result = new Result<BaseMaterialFileClassification>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.baseMaterialFileClassificationService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<BaseMaterialFileClassification> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BaseMaterialFileClassification> result = new Result<BaseMaterialFileClassification>();
		BaseMaterialFileClassification baseMaterialFileClassification = baseMaterialFileClassificationService.getById(id);
		if(baseMaterialFileClassification==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(baseMaterialFileClassification);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<BaseMaterialFileClassification> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              BaseMaterialFileClassification baseMaterialFileClassification = JSON.parseObject(deString, BaseMaterialFileClassification.class);
              queryWrapper = QueryGenerator.initQueryWrapper(baseMaterialFileClassification, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<BaseMaterialFileClassification> pageList = baseMaterialFileClassificationService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "物料档案分类列表");
      mv.addObject(NormalExcelConstants.CLASS, BaseMaterialFileClassification.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("物料档案分类列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<BaseMaterialFileClassification> listBaseMaterialFileClassifications = ExcelImportUtil.importExcel(file.getInputStream(), BaseMaterialFileClassification.class, params);
              for (BaseMaterialFileClassification baseMaterialFileClassificationExcel : listBaseMaterialFileClassifications) {
                  baseMaterialFileClassificationService.save(baseMaterialFileClassificationExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listBaseMaterialFileClassifications.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
