package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.BaseMaterialFileModular;
import org.jeecg.modules.basedata.service.IBaseMaterialFileModularService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 物料档案模块
 * @author： jeecg-boot
 * @date：   2021-11-16
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/baseMaterialFileModular")
@Slf4j
public class BaseMaterialFileModularController {
	@Autowired
	private IBaseMaterialFileModularService baseMaterialFileModularService;
	
	/**
	  * 分页列表查询
	 * @param baseMaterialFileModular
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<BaseMaterialFileModular>> queryPageList(BaseMaterialFileModular baseMaterialFileModular,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<BaseMaterialFileModular>> result = new Result<IPage<BaseMaterialFileModular>>();
		QueryWrapper<BaseMaterialFileModular> queryWrapper = QueryGenerator.initQueryWrapper(baseMaterialFileModular, req.getParameterMap());
		Page<BaseMaterialFileModular> page = new Page<BaseMaterialFileModular>(pageNo, pageSize);
		IPage<BaseMaterialFileModular> pageList = baseMaterialFileModularService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param baseMaterialFileModular
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<BaseMaterialFileModular> add(@RequestBody BaseMaterialFileModular baseMaterialFileModular) {
		Result<BaseMaterialFileModular> result = new Result<BaseMaterialFileModular>();
		try {
			baseMaterialFileModularService.save(baseMaterialFileModular);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param baseMaterialFileModular
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<BaseMaterialFileModular> edit(@RequestBody BaseMaterialFileModular baseMaterialFileModular) {
		Result<BaseMaterialFileModular> result = new Result<BaseMaterialFileModular>();
		BaseMaterialFileModular baseMaterialFileModularEntity = baseMaterialFileModularService.getById(baseMaterialFileModular.getId());
		if(baseMaterialFileModularEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = baseMaterialFileModularService.updateById(baseMaterialFileModular);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<BaseMaterialFileModular> delete(@RequestParam(name="id",required=true) String id) {
		Result<BaseMaterialFileModular> result = new Result<BaseMaterialFileModular>();
		BaseMaterialFileModular baseMaterialFileModular = baseMaterialFileModularService.getById(id);
		if(baseMaterialFileModular==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = baseMaterialFileModularService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<BaseMaterialFileModular> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<BaseMaterialFileModular> result = new Result<BaseMaterialFileModular>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.baseMaterialFileModularService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<BaseMaterialFileModular> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BaseMaterialFileModular> result = new Result<BaseMaterialFileModular>();
		BaseMaterialFileModular baseMaterialFileModular = baseMaterialFileModularService.getById(id);
		if(baseMaterialFileModular==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(baseMaterialFileModular);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<BaseMaterialFileModular> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              BaseMaterialFileModular baseMaterialFileModular = JSON.parseObject(deString, BaseMaterialFileModular.class);
              queryWrapper = QueryGenerator.initQueryWrapper(baseMaterialFileModular, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<BaseMaterialFileModular> pageList = baseMaterialFileModularService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "物料档案模块列表");
      mv.addObject(NormalExcelConstants.CLASS, BaseMaterialFileModular.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("物料档案模块列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<BaseMaterialFileModular> listBaseMaterialFileModulars = ExcelImportUtil.importExcel(file.getInputStream(), BaseMaterialFileModular.class, params);
              for (BaseMaterialFileModular baseMaterialFileModularExcel : listBaseMaterialFileModulars) {
                  baseMaterialFileModularService.save(baseMaterialFileModularExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listBaseMaterialFileModulars.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
