package org.jeecg.modules.basedata.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.BaseSalesStatus;
import org.jeecg.modules.basedata.model.BaseSalesStatusTree;
import org.jeecg.modules.basedata.service.IBaseSalesStatusService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 销售状态
 * @author： jeecg-boot
 * @date：   2021-11-12
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/baseSalesStatus")
@Slf4j
public class BaseSalesStatusController {
	@Autowired
	private IBaseSalesStatusService baseSalesStatusService;
	 @Autowired
	 private ISysSerialPatternService sysSerialPatternService;
	
	/**
	  * 分页列表查询
	 * @param baseSalesStatus
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<BaseSalesStatus>> queryPageList(BaseSalesStatus baseSalesStatus,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<BaseSalesStatus>> result = new Result<IPage<BaseSalesStatus>>();
		QueryWrapper<BaseSalesStatus> queryWrapper = QueryGenerator.initQueryWrapper(baseSalesStatus, req.getParameterMap());
		Page<BaseSalesStatus> page = new Page<BaseSalesStatus>(pageNo, pageSize);
		IPage<BaseSalesStatus> pageList = baseSalesStatusService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 获取树形数据
	* @Date 2021/11/12 13:55
	* @Param [pkOrg]
	* @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.basedata.model.BaseSalesStatusTree>>
	*/
	 @GetMapping(value = "/getTreeList")
	 public Result<List<BaseSalesStatusTree>> getTreeList(String pkOrg) {
		 Result<List<BaseSalesStatusTree>> result = new Result<>();
		 try {
			 QueryWrapper<BaseSalesStatus> queryWrapper = new QueryWrapper<>();
			 queryWrapper.lambda().eq(BaseSalesStatus::getDelFlag,"0");
			 queryWrapper.lambda().eq(BaseSalesStatus::getPkOrg,pkOrg);
			 queryWrapper.lambda().orderByAsc(BaseSalesStatus::getSort);
			 List<BaseSalesStatus> list = baseSalesStatusService.list(queryWrapper);
			 List<BaseSalesStatusTree> treeList=new ArrayList<>();
			 getTree(treeList,list,null);
			 findLeaf(treeList);
			 result.success("操作成功");
			 result.setResult(treeList);
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

	 //处理树形格式
	 private void getTree(List<BaseSalesStatusTree> treeList, List<BaseSalesStatus> fdlist, BaseSalesStatusTree temp) {
		 int i = 0;
		 for (BaseSalesStatus baseSalesStatus : fdlist) {
			 String tempPid = baseSalesStatus.getParentId();
			 BaseSalesStatusTree tree = new BaseSalesStatusTree(baseSalesStatus);
			 if(temp == null) {
				 if(oConvertUtils.isEmpty(tempPid)){
					 treeList.add(tree);
				 }
				 if(!tree.getIsLeaf()) {
					 getTree(treeList, fdlist, tree);
				 }
			 }else if(tempPid != null && tempPid.equals(temp.getKey())){
				 temp.getChildren().add(tree);
				 if(!tree.getIsLeaf()) {
					 getTree(treeList, fdlist, tree);
				 }
			 }

		 }
	 }

	 //处理尾端
	 private void findLeaf(List<BaseSalesStatusTree> treeList){
		 for(BaseSalesStatusTree baseSalesStatusTree : treeList){
			 if(baseSalesStatusTree.getChildren().isEmpty()){
				 baseSalesStatusTree.setIsLeaf(true);
			 }else{
				 findLeaf(baseSalesStatusTree.getChildren());
			 }
		 }

	 }
	
	/**
	  *   添加
	 * @param baseSalesStatus
	 * @return
	 */
	 @PostMapping(value = "/add")
     @RequiresPermissions("sales-status:add")
     public Result<BaseSalesStatus> add(@RequestBody BaseSalesStatus baseSalesStatus) {
		Result<BaseSalesStatus> result = new Result<BaseSalesStatus>();
		try {
			// 设置编码
			CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(baseSalesStatus, "base_sales_status", "code", true);
			if(!nextSerial.isSucceed()){
				throw new RuntimeException("获取编号失败");
			}
			baseSalesStatus.setCode(nextSerial.getContent());
			baseSalesStatusService.save(baseSalesStatus);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param baseSalesStatus
	 * @return
	 */
	@PutMapping(value = "/edit")
    @RequiresPermissions("sales-status:update")
	public Result<BaseSalesStatus> edit(@RequestBody BaseSalesStatus baseSalesStatus) {
		Result<BaseSalesStatus> result = new Result<BaseSalesStatus>();
		BaseSalesStatus baseSalesStatusEntity = baseSalesStatusService.getById(baseSalesStatus.getId());
		if(baseSalesStatusEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = baseSalesStatusService.updateById(baseSalesStatus);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
//	@DeleteMapping(value = "/delete")
//	public Result<BaseSalesStatus> delete(@RequestParam(name="id",required=true) String id) {
//		Result<BaseSalesStatus> result = new Result<BaseSalesStatus>();
//		BaseSalesStatus baseSalesStatus = baseSalesStatusService.getById(id);
//		if(baseSalesStatus==null) {
//			result.error500("未找到对应实体");
//		}else {
//			boolean ok = baseSalesStatusService.removeById(id);
//			if(ok) {
//				result.success("删除成功!");
//			}
//		}
//
//		return result;
//	}


	 /**
	 * @Author chenchuang
	 * @Description //TODO 根据id逻辑删除
	 * @Date 2021/11/12 13:59
	 * @Param [id]
	 * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.basedata.entity.BaseSalesStatus>
	 */
	 @DeleteMapping(value = "/delete")
     @RequiresPermissions("sales-status:delete")
	 public Result<BaseSalesStatus> delete(@RequestParam(name="id",required=true) String id) {
		 Result<BaseSalesStatus> result = new Result<BaseSalesStatus>();
		 BaseSalesStatus baseSalesStatus = baseSalesStatusService.getById(id);
		 if(baseSalesStatus==null) {
			 result.error500("未找到对应实体");
		 }else {
			 baseSalesStatus.setDelFlag("1");
			 boolean ok = baseSalesStatusService.updateById(baseSalesStatus);
			 if(ok) {
				 result.success("删除成功!");
			 }
		 }

		 return result;
	 }
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<BaseSalesStatus> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<BaseSalesStatus> result = new Result<BaseSalesStatus>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.baseSalesStatusService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<BaseSalesStatus> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BaseSalesStatus> result = new Result<BaseSalesStatus>();
		BaseSalesStatus baseSalesStatus = baseSalesStatusService.getById(id);
		if(baseSalesStatus==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(baseSalesStatus);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<BaseSalesStatus> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              BaseSalesStatus baseSalesStatus = JSON.parseObject(deString, BaseSalesStatus.class);
              queryWrapper = QueryGenerator.initQueryWrapper(baseSalesStatus, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<BaseSalesStatus> pageList = baseSalesStatusService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "销售状态列表");
      mv.addObject(NormalExcelConstants.CLASS, BaseSalesStatus.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("销售状态列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<BaseSalesStatus> listBaseSalesStatuss = ExcelImportUtil.importExcel(file.getInputStream(), BaseSalesStatus.class, params);
              for (BaseSalesStatus baseSalesStatusExcel : listBaseSalesStatuss) {
                  baseSalesStatusService.save(baseSalesStatusExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listBaseSalesStatuss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
