package org.jeecg.modules.contract.file.controller;

import java.util.*;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oaEntity.OaEntity;
import org.jeecg.common.util.OkHttpClientUtil;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.BaseArchivesProjectApproval;
import org.jeecg.modules.basedata.entity.FdCustomer;
import org.jeecg.modules.basedata.service.BaseArchivesProjectApprovalService;
import org.jeecg.modules.basedata.service.IFdCustomerService;
import org.jeecg.modules.contract.file.entity.ContractFile;
import org.jeecg.modules.contract.file.service.IContractFileService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.service.ISysDictService;
import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 合同档案
 * @author： jeecg-boot
 * @date：   2021-11-17
 * @version： V1.0
 */
@RestController
@RequestMapping("/contract.file/contractFile")
@Slf4j
public class ContractFileController {
	@Autowired
	private IContractFileService contractFileService;
	@Autowired
	private IFdCustomerService iFdCustomerService;
	@Autowired
	private ISysDictService iSysDictService;
	@Autowired
	private ISysSerialPatternService sysSerialPatternService;
	@Autowired
	private BaseArchivesProjectApprovalService baseArchivesProjectApprovalService;

//	 @Autowired
//	 private OkHttpClient okHttpClient;
	
	/**
	  * 分页列表查询
	 * @param contractFile
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<ContractFile>> queryPageList(ContractFile contractFile,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<ContractFile>> result = new Result<IPage<ContractFile>>();
		QueryWrapper<ContractFile> queryWrapper = QueryGenerator.initQueryWrapper(contractFile, req.getParameterMap());
		queryWrapper.lambda().eq(ContractFile::getDelFlag,"0");
		queryWrapper.lambda().orderByDesc(ContractFile::getCreateTime);
		Page<ContractFile> page = new Page<ContractFile>(pageNo, pageSize);
		IPage<ContractFile> pageList = contractFileService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);


		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 分页查询
	* @Date 2021/11/25 13:35
	* @Param [contractFile, pageNo, pageSize, req]
	* @return org.jeecg.common.api.vo.Result<com.baomidou.mybatisplus.core.metadata.IPage<org.jeecg.modules.contract.file.entity.ContractFile>>
	*/
	 @GetMapping(value = "/getPageList")
	 public Result<IPage<ContractFile>> getPageList(ContractFile contractFile,
													  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													  HttpServletRequest req) {
		 Result<IPage<ContractFile>> result = new Result<IPage<ContractFile>>();
		 String pkOrg="";
		 if(StringUtils.isNotBlank(contractFile.getPkOrg())){
			 pkOrg=contractFile.getPkOrg();
			 contractFile.setPkOrg(null);
		 }
		 String code="";
		 if(StringUtils.isNotBlank(contractFile.getCode())){
		 	code=contractFile.getCode();
		 	contractFile.setCode(null);
		 }
		 QueryWrapper<ContractFile> queryWrapper = QueryGenerator.initQueryWrapper(contractFile, req.getParameterMap());
		 queryWrapper.eq("cf.del_flag","0");
		 queryWrapper.eq("cf.pk_org",pkOrg);
		 if(StringUtils.isNotBlank(code)){
			 queryWrapper.eq("cf.code",code);
		 }
		 queryWrapper.orderByDesc("cf.create_time");
		 Page<ContractFile> page = new Page<ContractFile>(pageNo, pageSize);
		 IPage<ContractFile> pageList = contractFileService.getPageList(page, queryWrapper);
		 getDictName(pageList.getRecords());
		 result.setSuccess(true);
		 result.setResult(pageList);
		 return result;
	 }

	 //获取数据字典数据
	 public void getDictName(List<ContractFile> contractFileList){
		if(contractFileList!=null&&contractFileList.size()>0){
			//获取里程碑类型名称
			//里程碑类型数据字典数据
			List<Map<String,Object>> dictList= iSysDictService.queryDictItemsByCode("base_archives_milestone_type");
			if(dictList!=null){
				for(ContractFile contractFile:contractFileList){
					for(Map<String,Object> dict:dictList){
						if(contractFile.getMilestoneId().equals(dict.get("value").toString())){
							contractFile.setMilestoneName(dict.get("text").toString());
						}
					}
				}
			}

		}
	 }

	/**
	  *   添加
	 * @param contractFile
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<ContractFile> add(@RequestBody ContractFile contractFile) {
		Result<ContractFile> result = new Result<ContractFile>();
		try {
			contractFileService.save(contractFile);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 主表和子表数据一起保存
	* @Date 2021/11/17 16:41
	* @Param [contractFile]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @PostMapping(value = "/saveMainAndChild")
	 public Result<ContractFile> saveMainAndChild(@RequestBody ContractFile contractFile) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 try {
			 // 设置编码
			 CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(contractFile, "contract_file", "code", true);
			 if(!nextSerial.isSucceed()){
				 throw new RuntimeException("获取编号失败");
			 }
			 contractFile.setCode(nextSerial.getContent());
			 contractFileService.saveMainAndChild(contractFile);
			 result.success("添加成功！");
		 } catch (Exception e) {
			 e.printStackTrace();
			 log.info(e.getMessage());
			 result.error500("操作失败");
		 }
		 return result;
	 }
	
	/**
	  *  编辑
	 * @param contractFile
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<ContractFile> edit(@RequestBody ContractFile contractFile) {
		Result<ContractFile> result = new Result<ContractFile>();
		ContractFile contractFileEntity = contractFileService.getById(contractFile.getId());
		if(contractFileEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = contractFileService.updateById(contractFile);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 主表和子表一起修改
	* @Date 2021/11/17 17:19
	* @Param [contractFile]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @PutMapping(value = "/updateMainAndChild")
	 public Result<ContractFile> updateMainAndChild(@RequestBody ContractFile contractFile) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 ContractFile contractFileEntity = contractFileService.getById(contractFile.getId());
		 if(contractFileEntity==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 contractFileService.updateMainAndChild(contractFile);
				 result.success("修改成功");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("修改失败："+e.getMessage());
			 }
		 }
		 return result;
	 }
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<ContractFile> delete(@RequestParam(name="id",required=true) String id) {
		Result<ContractFile> result = new Result<ContractFile>();
		ContractFile contractFile = contractFileService.getById(id);
		if(contractFile==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = contractFileService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 主表与子表一起删除
	* @Date 2021/11/17 17:29
	* @Param [id]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @DeleteMapping(value = "/deleteMainAndChild")
	 public Result<ContractFile> deleteMainAndChild(@RequestParam(name="id",required=true) String id) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 ContractFile contractFile = contractFileService.getById(id);
		 if(contractFile==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 contractFileService.deleteMainAndChild(contractFile);
				 result.success("删除成功");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("删除失败："+e.getMessage());
			 }
		 }
		 return result;
	 }
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<ContractFile> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<ContractFile> result = new Result<ContractFile>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.contractFileService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<ContractFile> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ContractFile> result = new Result<ContractFile>();
		ContractFile contractFile = contractFileService.getById(id);
		if(contractFile==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(contractFile);
			result.setSuccess(true);
		}
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 根据主表id查询主子表数据
	* @Date 2021/11/17 17:31
	* @Param [id]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
	*/
	 @GetMapping(value = "/queryMainAndChildById")
	 public Result<ContractFile> queryMainAndChildById(@RequestParam(name="id",required=true) String id) {
		 Result<ContractFile> result = new Result<ContractFile>();
		 ContractFile contractFile = contractFileService.getById(id);
		 if(contractFile==null) {
			 result.error500("未找到对应实体");
		 }else {
		 	//获取客户名称
			 FdCustomer fdCustomer= iFdCustomerService.getById(contractFile.getFdCustomerId());
			 if(fdCustomer!=null){
				 contractFile.setFdCustomerName(fdCustomer.getName());
			 }
		 	//获取所有子表数据
			 contractFileService.queryMainAndChildById(contractFile);
			 result.setResult(contractFile);
			 result.setSuccess(true);
		 }
		 return result;
	 }

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<ContractFile> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              ContractFile contractFile = JSON.parseObject(deString, ContractFile.class);
              queryWrapper = QueryGenerator.initQueryWrapper(contractFile, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<ContractFile> pageList = contractFileService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "合同档案列表");
      mv.addObject(NormalExcelConstants.CLASS, ContractFile.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("合同档案列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<ContractFile> listContractFiles = ExcelImportUtil.importExcel(file.getInputStream(), ContractFile.class, params);
              for (ContractFile contractFileExcel : listContractFiles) {
                  contractFileService.save(contractFileExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listContractFiles.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 提交流程
  * @Date 2021/11/26 16:15
  * @Param [id]
  * @return org.jeecg.common.api.vo.Result
  */
  @GetMapping(value = "/submitProcess")
  public Result submitProcess(@RequestParam(name="id",required=true) String id){
  	  Result result=new Result();
	  ContractFile contractFile = contractFileService.getById(id);
	  if(contractFile==null) {
		  result.error500("未找到对应实体");
	  }else {
		  OkHttpClientUtil okHttpClientUtil = new OkHttpClientUtil();
		  //参数
		  OaEntity oaEntity = new OaEntity();
		  oaEntity.setUrl("/actBusiness/external/apply");
		  SysUser user = (SysUser) SecurityUtils.getSubject().getPrincipal();
		  oaEntity.setExternalUsername(user.getUsername());
//		  oaEntity.setProcDefId("公用测试流程:1:182509");
		  oaEntity.setProcDefId("ht合同审批:1:192507");
		  oaEntity.setTableName("外部表单contract-file");
		  //获取项目立项名称
		  BaseArchivesProjectApproval projectApproval= baseArchivesProjectApprovalService.getById(contractFile.getProId());
		  if(projectApproval!=null){
			  oaEntity.setTitle(projectApproval.getName());
		  }
		  oaEntity.setTableId(id);
		  //调用Oa接口
		  JSONObject jsonObject = okHttpClientUtil.getPost(oaEntity);
		  if (jsonObject.get("success").toString().equals("true")) {
			  result.success("提交成功");
		  } else {
			  result.setSuccess(false);
			  result.setMessage("提交失败");
		  }
	  }
	  return result;
  }

  /**
  * @Author chenchuang
  * @Description //TODO 根据id查询合同数据
  * @Date 2021/11/30 11:55
  * @Param [id]
  * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.contract.file.entity.ContractFile>
  */
  @GetMapping(value = "/getContractFileById")
  public Result<ContractFile> getContractFileById(@RequestParam(name="id",required=true) String id){
  	Result<ContractFile> result =new Result();
	  ContractFile contractFile = contractFileService.getById(id);
	  if(contractFile==null) {
		  result.error500("未找到对应实体");
	  }else {

		  //查询主表详情数据
		  contractFile=contractFileService.getContractFileById(id);
		  //获取所有子表数据
		  contractFileService.queryMainAndChildById(contractFile);
		  result.setResult(contractFile);
		  result.setSuccess(true);
	  }
  	return result;
  }

}
