package org.jeecg.modules.contract.file.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.jeecg.modules.contract.file.entity.ContractFile;
import org.jeecg.modules.contract.file.entity.ContractFileAndBusiness;
import org.jeecg.modules.contract.file.entity.ContractFileModular;
import org.jeecg.modules.contract.file.entity.ContractFileProduct;
import org.jeecg.modules.contract.file.mapper.ContractFileMapper;
import org.jeecg.modules.contract.file.service.IContractFileAndBusinessService;
import org.jeecg.modules.contract.file.service.IContractFileModularService;
import org.jeecg.modules.contract.file.service.IContractFileProductService;
import org.jeecg.modules.contract.file.service.IContractFileService;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 合同档案
 * @author： jeecg-boot
 * @date：   2021-11-17
 * @version： V1.0
 */
@Service
public class ContractFileServiceImpl extends ServiceImpl<ContractFileMapper, ContractFile> implements IContractFileService {

    @Resource
    ContractFileMapper contractFileMapper;
    @Resource
    IContractFileProductService iContractFileProductService;
    @Resource
    IContractFileModularService iContractFileModularService;
    @Resource
    IContractFileAndBusinessService iContractFileAndBusinessService;

    @Override
    public IPage<ContractFile> getPageList(IPage<ContractFile> page, QueryWrapper<ContractFile> queryWrapper) {
        return contractFileMapper.getPageList(page,queryWrapper);
    }

    @Override
    public void saveMainAndChild(ContractFile contractFile) {
        //新增主表
        contractFileMapper.insert(contractFile);
        //新增所有子表
        saveChildAll(contractFile);
    }

    @Override
    public void updateMainAndChild(ContractFile contractFile) {
        //修改主表
        contractFileMapper.updateById(contractFile);
        //删除所有子表
        deleteChildAll(contractFile);
        //新增所有子表
        saveChildAll(contractFile);

    }

    @Override
    public void deleteMainAndChild(ContractFile contractFile) {
        //主表删除
        contractFile.setDelFlag("1");
        contractFileMapper.updateById(contractFile);
        //删除所有子表
        deleteChildAll(contractFile);
    }

    @Override
    public void queryMainAndChildById(ContractFile contractFile) {
        //查询合同产品数据
        QueryWrapper<ContractFileProduct> contractFileProductQueryWrapper=new QueryWrapper<>();
        contractFileProductQueryWrapper.lambda().eq(ContractFileProduct::getContractFileId,contractFile.getId());
        contractFileProductQueryWrapper.lambda().eq(ContractFileProduct::getDelFlag,"0");
        contractFileProductQueryWrapper.lambda().orderByAsc(ContractFileProduct::getSort);
        List<ContractFileProduct> contractFileProductList=iContractFileProductService.list(contractFileProductQueryWrapper);
        //查询合同产品模块数据
        QueryWrapper<ContractFileModular> contractFileModularQueryWrapper=new QueryWrapper<>();
        contractFileModularQueryWrapper.lambda().eq(ContractFileModular::getContractFileId,contractFile.getId());
        contractFileModularQueryWrapper.lambda().eq(ContractFileModular::getDelFlag,"0");
        contractFileModularQueryWrapper.lambda().orderByAsc(ContractFileModular::getSort);
        List<ContractFileModular> contractFileModularList=iContractFileModularService.list(contractFileModularQueryWrapper);
        //查询合同收款计划数据
        QueryWrapper<ContractFileAndBusiness> contractFileAndBusinessQueryWrapper=new QueryWrapper<>();
        contractFileAndBusinessQueryWrapper.lambda().eq(ContractFileAndBusiness::getContractFileId,contractFile.getId());
        contractFileAndBusinessQueryWrapper.lambda().eq(ContractFileAndBusiness::getDelFlag,"0");
        contractFileAndBusinessQueryWrapper.lambda().orderByAsc(ContractFileAndBusiness::getSort);
        List<ContractFileAndBusiness> contractFileAndBusinessList=iContractFileAndBusinessService.list(contractFileAndBusinessQueryWrapper);
        //处理合同产品数据
        //判空
        if(contractFileProductList!=null&&contractFileProductList.size()>0){
            if(contractFileModularList!=null&&contractFileModularList.size()>0){
                //循环
                for(ContractFileProduct product:contractFileProductList){
                    List<ContractFileModular> modularList=new ArrayList<>();//存取对应产品的模块
                    for(ContractFileModular modular:contractFileModularList){
                        //根据产品id找到对应所属的模块
                        if(product.getId().equals(modular.getContractFileProductId())){
                            modularList.add(modular);
                        }
                    }
                    //找到模块放入到当前循环的产品中
                    product.setContractFileModularList(modularList);
                }
            }
            //把产品放到合同主表中
            contractFile.setContractFileProductList(contractFileProductList);
        }
        //把收款条件放到合同主表中
        if(contractFileAndBusinessList!=null&&contractFileAndBusinessList.size()>0){
            contractFile.setContractFileAndBusinessList(contractFileAndBusinessList);
        }
    }


    /**新增所有子表*/
    public void saveChildAll(ContractFile contractFile){
        //新增合同产品表
        if(contractFile.getContractFileProductList()!=null&&contractFile.getContractFileProductList().size()>0){
            Integer sort=1;
            for(ContractFileProduct contractFileProduct:contractFile.getContractFileProductList()){
                contractFileProduct.setId(null);
                contractFileProduct.setContractFileId(contractFile.getId());
                contractFileProduct.setSort(sort);
                sort++;
            }
            //保存合同产品
            iContractFileProductService.saveBatch(contractFile.getContractFileProductList());
            //新增合同产品模块表
            List<ContractFileModular> contractFileModularList=new ArrayList<>();//储存合同产品模块表集合
            //循环获取
            for(ContractFileProduct contractFileProduct:contractFile.getContractFileProductList()){
                String productId=contractFileProduct.getId();//合同产品id
                //获取合同产品模块
                if(contractFileProduct.getContractFileModularList()!=null&&contractFileProduct.getContractFileModularList().size()>0){
                    Integer modularSort=1;
                    for(ContractFileModular contractFileModular:contractFileProduct.getContractFileModularList()){
                        contractFileModular.setId(null);
                        contractFileModular.setContractFileId(contractFile.getId());
                        contractFileModular.setContractFileProductId(productId);
                        contractFileModular.setSort(modularSort);
                        modularSort++;
                        contractFileModularList.add(contractFileModular);
                    }
                }
            }
            //判断是否存在合同产品模块数据
            if(contractFileModularList.size()>0){
                //保存合同产品模块
                iContractFileModularService.saveBatch(contractFileModularList);
            }
        }
        //新增合同收款计划
        if(contractFile.getContractFileAndBusinessList()!=null&&contractFile.getContractFileAndBusinessList().size()>0){
            Integer busSort=1;
            for(ContractFileAndBusiness contractFileAndBusiness:contractFile.getContractFileAndBusinessList()){
                contractFileAndBusiness.setId(null);
                contractFileAndBusiness.setContractFileId(contractFile.getId());
                contractFileAndBusiness.setSort(busSort);
                busSort++;
            }
            //保存合同收款计划
            iContractFileAndBusinessService.saveBatch(contractFile.getContractFileAndBusinessList());
        }
    }

    /**删除所有子表*/
    public void deleteChildAll(ContractFile contractFile){
        //删除合同产品表
        QueryWrapper<ContractFileProduct> contractFileProductQueryWrapper=new QueryWrapper<>();
        contractFileProductQueryWrapper.lambda().eq(ContractFileProduct::getContractFileId,contractFile.getId());
        ContractFileProduct contractFileProduct=new ContractFileProduct();
        contractFileProduct.setDelFlag("1");
        iContractFileProductService.update(contractFileProduct,contractFileProductQueryWrapper);
        //删除合同产品模板表
        QueryWrapper<ContractFileModular> contractFileModularQueryWrapper=new QueryWrapper<>();
        contractFileModularQueryWrapper.lambda().eq(ContractFileModular::getContractFileId,contractFile.getId());
        ContractFileModular contractFileModular=new ContractFileModular();
        contractFileModular.setDelFlag("1");
        iContractFileModularService.update(contractFileModular,contractFileModularQueryWrapper);
        //删除合同收款计划表
        QueryWrapper<ContractFileAndBusiness> contractFileAndBusinessQueryWrapper=new QueryWrapper<>();
        contractFileAndBusinessQueryWrapper.lambda().eq(ContractFileAndBusiness::getContractFileId,contractFile.getId());
        ContractFileAndBusiness contractFileAndBusiness=new ContractFileAndBusiness();
        contractFileAndBusiness.setDelFlag("1");
        iContractFileAndBusinessService.update(contractFileAndBusiness,contractFileAndBusinessQueryWrapper);
    }


    @Override
    public ContractFile getContractFileById(String id) {
        ContractFile contractFile=contractFileMapper.getContractFileById(id);
        if(contractFile!=null){
            //获取产品
            QueryWrapper<ContractFileProduct> productQueryWrapper=new QueryWrapper<>();
            productQueryWrapper.lambda().eq(ContractFileProduct::getDelFlag,"0");
            productQueryWrapper.lambda().eq(ContractFileProduct::getContractFileId,contractFile.getId());
            productQueryWrapper.lambda().orderByAsc(ContractFileProduct::getSort);
            List<ContractFileProduct> productList=iContractFileProductService.list(productQueryWrapper);
            //获取模块
//            QueryWrapper<ContractFileModular>
        }
        return contractFileMapper.getContractFileById(id);
    }

}
