package org.jeecg.modules.expense.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.dto.expense.ExpenseSlipAddReqDTO;
import org.jeecg.common.dto.expense.ExpenseSlipProjectAddReqDTO;
import org.jeecg.common.dto.expense.ExpenseSlipProjectListRespDTO;
import org.jeecg.common.dto.expense.ExpenseSlipRespDTO;
import org.jeecg.common.dto.expense.SlipUpdateFile;
import org.jeecg.common.dto.expense.SlipUpdateFileRespDTO;
import org.jeecg.modules.expense.entity.ReExpenseSlip;
import org.jeecg.modules.expense.entity.ReExpenseSlipProject;
import org.jeecg.modules.expense.entity.ReExpenseSlipUpdateFile;
import org.jeecg.modules.expense.mapper.ReExpenseSlipMapper;
import org.jeecg.modules.expense.service.IReExpenseSlipUpdateFileService;
import org.jeecg.modules.expense.service.ReExpenseSlipProjectService;
import org.jeecg.modules.expense.service.ReExpenseSlipService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 费用报销单
 * @author： jeecg-boot
 * @date：   2021-03-03
 * @version： V1.0
 */
@Service
public class ReExpenseSlipServiceImpl extends ServiceImpl<ReExpenseSlipMapper, ReExpenseSlip> implements ReExpenseSlipService {
    @Autowired
    private ReExpenseSlipProjectService reExpenseSlipProjectService;
    @Autowired
    private IReExpenseSlipUpdateFileService reExpenseSlipUpdateFileService;

    /**
     * @desc 新增
     */
    @Override
    public void add(ExpenseSlipAddReqDTO reqDTO) {
        //保存主表
        ReExpenseSlip reExpenseSlip = new ReExpenseSlip(reqDTO);
         save(reExpenseSlip);
         //保存子表
        saveChildren(reqDTO, reExpenseSlip.getId());
    }

    /**
     * @desc 编辑
     */
    @Override
    public boolean edit(ExpenseSlipAddReqDTO reqDTO) {
        //修改主表
        updateById(new ReExpenseSlip(reqDTO));
        //删除子表
        deleteChildren(reqDTO.getId());
        //保存子表
        saveChildren(reqDTO, reqDTO.getId());
        return true;
    }

    /**
     * @desc 根据id删除主子表
     */
    @Override
    public boolean deleteById(ReExpenseSlip reExpenseSlip) {
        reExpenseSlip.setDelFlag("1");
        updateById(reExpenseSlip);
        //删除子表
        deleteChildren(reExpenseSlip.getId());
        return true;
    }

    /**
     * @desc 根据id查询主子表
     */
    @Override
    public ExpenseSlipRespDTO getListById(String id) {
        ReExpenseSlip slip = getById(id);
        ExpenseSlipRespDTO respDTO = new ExpenseSlipRespDTO(slip);
        //设置子表
        respDTO.setDetailList(getChildren(id));
        return respDTO;
    }



    /**
     * @desc 获取子表
     */
    private List<ExpenseSlipProjectListRespDTO> getChildren(String id) {
        QueryWrapper<ReExpenseSlipProject> queryWrapper = new QueryWrapper();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("expense_id", id);
        queryWrapper.orderByAsc("sort");
        List<ReExpenseSlipProject> list = reExpenseSlipProjectService.list(queryWrapper);

        //查询附件
        QueryWrapper<ReExpenseSlipUpdateFile> fileQueryWrapper = new QueryWrapper<>();
        fileQueryWrapper.eq("expense_id", id);
        fileQueryWrapper.eq("del_flag","0");
        List<ReExpenseSlipUpdateFile> updateFiles = reExpenseSlipUpdateFileService.list(fileQueryWrapper);
        Map<String, List<ReExpenseSlipUpdateFile>> updateFileMap = updateFiles.stream().collect(Collectors.groupingBy(t -> t.getExpenseProjectId()));
        List<ExpenseSlipProjectListRespDTO> respDTOS = Lists.newArrayList();
        for(ReExpenseSlipProject project : list){
            ExpenseSlipProjectListRespDTO respDTO = new ExpenseSlipProjectListRespDTO(project);

            respDTO.setFiles(getUpdateFiles(project.getId(), updateFileMap));

            respDTOS.add(respDTO);
        }
        return respDTOS;
    }

    /**
     * @desc 获取附件
     */
    private List<SlipUpdateFileRespDTO> getUpdateFiles(String id, Map<String, List<ReExpenseSlipUpdateFile>> updateFileMap) {
        List<ReExpenseSlipUpdateFile> updateFiles = updateFileMap.get(id);
        List<SlipUpdateFileRespDTO> respDTOS = Lists.newArrayList();
        if(!CollectionUtils.isEmpty(updateFiles)){
            for(ReExpenseSlipUpdateFile updateFile : updateFiles){
                respDTOS.add(new SlipUpdateFileRespDTO(updateFile));
            }
        }

        return respDTOS;
    }

    /**
     * @desc 删除子表
     */
    private void   deleteChildren(String id) {
        QueryWrapper<ReExpenseSlipProject> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("expense_id", id);
        queryWrapper.eq("del_flag","0");
        reExpenseSlipProjectService.update(new ReExpenseSlipProject("1"), queryWrapper);

        //删除附件
        QueryWrapper<ReExpenseSlipUpdateFile> fileQueryWrapper = new QueryWrapper<>();
        fileQueryWrapper.eq("expense_id", id);
        fileQueryWrapper.eq("del_flag","0");
        reExpenseSlipUpdateFileService.update(new ReExpenseSlipUpdateFile("1"), fileQueryWrapper);

    }

    /**
     * @desc 保存子表
     */
    private void saveChildren(ExpenseSlipAddReqDTO reqDTO, String id) {
        List<ExpenseSlipProjectAddReqDTO> detailList = reqDTO.getDetailList();
        List<ReExpenseSlipProject> list = Lists.newArrayList();
        int sort = 1;
        for(ExpenseSlipProjectAddReqDTO detail : detailList){
            ReExpenseSlipProject project = new ReExpenseSlipProject(detail);
            project.setExpenseId(id);
            project.setSort(sort++);

            reExpenseSlipProjectService.save(project);
            //保存附件
            saveUpdateFile(detail.getFiles(), project.getId(), id);
        }
    }

    /**
     * @desc 保存附件
     */
    private void saveUpdateFile(List<SlipUpdateFile> files, String projectId, String id) {
        if(!CollectionUtils.isEmpty(files)) {
            List<ReExpenseSlipUpdateFile> list = Lists.newArrayList();
            int sort = 1;
            for(SlipUpdateFile file : files) {
                //截取文件名称
                String[] filePath = file.getFileUrl().split("/");
                int index = filePath.length;
                String fileNameAfter = StringUtils.substringAfter(filePath[index - 1], ".");
                String fileNameBefore = StringUtils.substringBefore(filePath[index - 1], "_");
                String fileName = fileNameBefore + "." + fileNameAfter;
                ReExpenseSlipUpdateFile reExpenseSlipUpdateFile = new ReExpenseSlipUpdateFile();
                reExpenseSlipUpdateFile.setFileName(fileName);
                reExpenseSlipUpdateFile.setFileUrl(file.getFileUrl());
                reExpenseSlipUpdateFile.setExpenseId(id);
                reExpenseSlipUpdateFile.setExpenseProjectId(projectId);
                reExpenseSlipUpdateFile.setSort(++sort);
                list.add(reExpenseSlipUpdateFile);
            }
            reExpenseSlipUpdateFileService.saveBatch(list);
        }
    }
}
